/**
 * @module node_opcua_crypto
 */
import { Certificate, CertificatePEM } from "./common";
import { SubjectPublicKey } from "./crypto_explore_certificate";
import { DirectoryName } from "./asn1";
export declare type PublicKeyLength = 128 | 256 | 384 | 512;
/**
 * A structure exposing useful information about a certificate
 */
export interface CertificateInfo {
    /** the public key length in bits */
    publicKeyLength: PublicKeyLength;
    /** the date at which the certificate starts to be valid */
    notBefore: Date;
    /** the date after which the certificate is not valid any more */
    notAfter: Date;
    /** info about certificate owner */
    subject: DirectoryName;
    /** public key */
    publicKey: SubjectPublicKey;
}
export declare function coerceCertificate(certificate: Certificate | CertificatePEM): Certificate;
/**
 * @method exploreCertificateInfo
 * returns useful information about the certificate such as public key length, start date and end of validity date,
 * and CN
 * @param certificate the certificate to explore
 */
export declare function exploreCertificateInfo(certificate: Certificate | CertificatePEM): CertificateInfo;
