"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getBuiltInDataType = void 0;
const node_opcua_data_model_1 = require("node-opcua-data-model");
const node_opcua_status_code_1 = require("node-opcua-status-code");
const find_basic_datatype_1 = require("./find_basic_datatype");
function getBuiltInDataType(session, variableNodeId, callback) {
    if (typeof callback !== "function") {
        throw new Error("Expecting a callback");
    }
    let dataTypeId = null;
    const nodeToRead = {
        attributeId: node_opcua_data_model_1.AttributeIds.DataType,
        nodeId: variableNodeId
    };
    session.read(nodeToRead, (err, dataValue) => {
        if (err) {
            return callback(err);
        }
        /* istanbul ignore next */
        if (!dataValue) {
            return callback(new Error("Internal Error"));
        }
        /* istanbul ignore next */
        if (dataValue.statusCode.isNot(node_opcua_status_code_1.StatusCodes.Good)) {
            return callback(new Error("cannot read DataType Attribute " + dataValue.statusCode.toString()));
        }
        dataTypeId = dataValue.value.value;
        (0, find_basic_datatype_1.findBasicDataType)(session, dataTypeId, callback);
    });
}
exports.getBuiltInDataType = getBuiltInDataType;
const thenify = require("thenify");
exports.getBuiltInDataType = thenify.withCallback(exports.getBuiltInDataType);
//# sourceMappingURL=get_builtin_datatype.js.map